# ---------------------------------------------------------------------------
# @author Shota Inoue
# @date   2025-11-02
#
# Generate 3D Molecule structure by SMILES strings.
#
# Package dependencies:
#    - NumPy
#    - RDKit
#    - OpenBabel
#    - NetworkX
# ---------------------------------------------------------------------------

import argparse as ap
import fnmatch
from smiles2mol import __version__
from .process import process

# extension validator
def ext_validator(ptns):
    def extension(val):
        for ptn in ptns:
            if fnmatch.fnmatch(val, ptn):
                return val
        raise ap.ArgumentTypeError(f"{val} is an invalid extension file.")
    return extension
    
# negative values validator
def neg_validator(val):
    fval = int(val)
    if fval >= 0:
        return fval
    raise ap.ArgumentTypeError(f"{val!r} is not positive.")

# main function (command line parser)
def main():
    ptns_out = ["*.mol", "*.mol2", "*.pdb", "*.xyz", "*.com", "*.gjf", "*.orcainp", "*.svg"]
    parser = ap.ArgumentParser(
        prog="smiles2mol",
        description=(
            f"Chemical structure generator from SMILES strings.\n\n"
            "----------------------------------------------------\n"
            "author  : Shota Inoue\n"
            "license : MIT\n"
            "guide   : https://github.com/s-inoue0108/smiles2mol\n"
            "----------------------------------------------------"
        ),
        formatter_class=ap.RawTextHelpFormatter,
    )
    parser.add_argument("-v", "--version", action="version", version=f"%(prog)s {__version__}")
    parser.add_argument("-s", "--smiles", required=True, help="Input SMILES strings.")
    parser.add_argument("-o", "--output", default="molecule.mol", type=ext_validator(ptns_out), help="Output file name.")
    parser.add_argument("--iter", type=neg_validator, default=1000, help="Number of optimize iteration.")
    parser.add_argument("--render", action="store_true", help="Render chemical structure instead of export.")

    # execute
    args = parser.parse_args()
    process(args)

if __name__ == "__main__":
    main()



# ---------------------------------------------------------------------------
# @author Shota Inoue
# @date   2025-11-02
#
# Generate 3D Molecule structure by SMILES strings.
#
# Package dependencies:
#    - NumPy
#    - RDKit
#    - OpenBabel
#    - NetworkX
# ---------------------------------------------------------------------------

from .lib.io import SmilesIO, ChemicalFormatIO
from .lib.structure import Structure
from .lib.render import Renderer

# main function
def process(args):
    smio = SmilesIO(args.smiles)
    rdmol = smio.get_rdmol()
    sc = Structure(rdmol)
    
    best_mol = sc.generate3Drdmol(nconfs=args.iter)
    xyzstr = sc.get_xyzstr(best_mol)
    
    cfio = ChemicalFormatIO(xyzstr, output=args.output)
    
    if args.render:
        rdmol = cfio.get_rdmol()
        renderer = Renderer(rdmol)
        renderer.run()
    else:
        cfio.export()



import os
import sys
import site
import subprocess

def ensure_openbabel_wheel():
    if os.environ.get("LD_LIBRARY_PATH_INITIALIZED"):
        return

    paths = []
    paths += site.getsitepackages()
    paths += [site.getusersitepackages()]

    for p in paths:
        wheel_lib = os.path.join(p, "openbabel")
        if os.path.isdir(wheel_lib):
            os.environ["LD_LIBRARY_PATH"] = wheel_lib
            os.environ["LD_LIBRARY_PATH_INITIALIZED"] = "1"
            os.execv(sys.executable, [sys.executable] + sys.argv)
            break

ensure_openbabel_wheel()

try:
    from openbabel import pybel
except ImportError:
    import pybel
    
from rdkit import Chem

class SmilesIO:
    def __init__(self, smiles):
        self.smiles = smiles
        self.rdmol = self.gen_rdmol()
        
    def gen_rdmol(self):
        rdmol = Chem.MolFromSmiles(self.smiles)
        rdmol = Chem.AddHs(rdmol)
        return rdmol
    
    def get_rdmol(self):
        return self.rdmol

class ChemicalFormatIO:
    def __init__(self, xyzstr, output):
        self.xyzstr = xyzstr
        self.output = output
        self.filename = os.path.basename(self.output)
        self.basename = os.path.splitext(self.filename)[0]
        self.format = os.path.splitext(self.filename)[1][1:]
        
    def get_filename(self):
        return self.filename
    
    def get_basename(self):
        return self.basename
        
    def get_format(self):
        return self.format
        
    def get_rdmol(self):
        mol = pybel.readstring("xyz", self.xyzstr)
        mol_block = mol.write("mol").rstrip()
        rdmol = Chem.MolFromMolBlock(mol_block, removeHs=False)
        return rdmol
        
    def export(self):
        mol = pybel.readstring("xyz", self.xyzstr)
        mol.write(self.format, self.filename, overwrite=True)
        print(f"Generate chemical structure: {self.filename}")
        return self



import os
import sys
from importlib.resources import files
import numpy as np
import networkx as nx
from matplotlib import font_manager as fm
import matplotlib.pyplot as plt
import matplotlib.colors as mcl

from .structure import Structure

class Renderer:
    def __init__(self, rdmol):
        self.rdmol = rdmol
        
    # visualize
    def run(self):
        atom_colors = {
            "H": "lightgray",
            "B": "pink",
            "C": "gray",
            "N": "blue",
            "O": "red",
            "F": "cyan",
            "Si": "lightblue",
            "P": "darkorange",
            "S": "yellow",
            "Cl": "green",
            "Se": "sandybrown",
            "Br": "salmon",
            "I": "blueviolet",
            "Sc": "#E6E6F2",
            "Ti": "#B3B3CC",
            "V": "#A680B3",
            "Cr": "#8A99C7",
            "Mn": "#9C4FA3",
            "Fe": "orange",
            "Co": "#4D80FF",
            "Ni": "#4FB34F",
            "Cu": "#C78033",
            "Zn": "#7D7FB0",
            "Y": "#94FFFF",
            "Zr": "#94E0E0",
            "Nb": "#73C2C2",
            "Mo": "#619999",
            "Tc": "#4D8080",
            "Ru": "#4069E0",
            "Rh": "#4069E0",
            "Pd": "silver",
            "Ag": "#BFBFBF",
            "Cd": "#57A6C2",
            "La": "#70B0F2",
            "Hf": "#4D8080",
            "Ta": "#4D8080",
            "W": "#1F1F1F",
            "Re": "#404040",
            "Os": "#4D4D4D",
            "Ir": "#808080",
            "Pt": "silver",
            "Au": "gold",
            "Hg": "#B3B3B3",
            "Tl": "#A8534D",
            "Pb": "#565656",
            "Bi": "#9E4F48",
            "Po": "#993333",
            "At": "#760076",
            "Rn": "#3FDFFF",
            "*": "purple",
        }
        
        background = "#ffffff"
        foreground = "#000000"
        negative = "#9900fa"
        positive = "#152eff"

        arial_path = files("smiles2mol.assets") / "Arial.ttf"
        arial = fm.FontProperties(fname=arial_path)
        
        plt.rcParams["font.family"] = arial.get_name()
        plt.rcParams["mathtext.fontset"] = "cm"

        plt.rcParams["figure.facecolor"] = background
        plt.rcParams["axes.facecolor"]   = background
        plt.rcParams["text.color"] = foreground
        plt.rcParams["axes.labelcolor"] = foreground
        plt.rcParams["xtick.color"] = foreground
        plt.rcParams["ytick.color"] = foreground
        
        fig = plt.figure(figsize=(12, 8))
        ax = fig.add_subplot(111, projection="3d")
        fig.canvas.manager.set_window_title(f"smiles2mol | renderer")
        
        # get graph
        sc = Structure(self.rdmol)
        graph = sc.get_graph(self.rdmol)

        # get nodes coordinate
        pos = nx.get_node_attributes(graph, "pos")
        symbols = nx.get_node_attributes(graph, "symbol")

        # draw nodes
        for i, (x, y, z) in pos.items():
            size = 100 if symbols[i] == "H" else 300
            fsize = 16 if symbols[i] == "H" else 24
            color = atom_colors[symbols[i]]
            
            # display node
            ax.scatter(x, y, z, s=size, c=color)

        # draw edges
        nodes = graph.nodes(data=True)
        for i, j, data in graph.edges(data=True):
            symbol_i = nodes[i]["symbol"]
            symbol_j = nodes[j]["symbol"]
            
            x = [pos[i][0], pos[j][0]]
            y = [pos[i][1], pos[j][1]]
            z = [pos[i][2], pos[j][2]]
            
            if data["order"] == 3:
                ax.plot(x, y, z, color="gray", linewidth=7.5)
                bv = np.array([x[1] - x[0], y[1] - y[0], z[1] - z[0]])
                
                # normal vec
                tmp = np.array([0, 0, 1]) if not np.allclose(bv[:2], 0) else np.array([0, 1, 0])
                u = np.cross(bv, tmp)
                u /= np.linalg.norm(u)
                offset = 0.05
                for sft in [-offset, offset]:
                    s = np.array((x[0], y[0], z[0])) + sft * u
                    e = np.array((x[1], y[1], z[1])) + sft * u
                    ax.plot([s[0], e[0]], [s[1], e[1]], [s[2], e[2]], color=background, linewidth=1.5)
            elif data["order"] == 2:
                ax.plot(x, y, z, color="gray", linewidth=4.5)
                ax.plot(x, y, z, color=background, linewidth=1.5)
            elif data["order"] == 1.5:
                ax.plot(x, y, z, color="gray", linestyle=(0, (1, 1)), linewidth=4.5)
                ax.plot(x, y, z, color=background, linewidth=1.5)
            else:
                ax.plot(x, y, z, color="gray", linewidth=2)
        
        x_max = np.max([item[1][0] for item in pos.items()])
        x_min = np.min([item[1][0] for item in pos.items()])
        y_max = np.max([item[1][1] for item in pos.items()])
        y_min = np.min([item[1][1] for item in pos.items()])
        z_max = np.max([item[1][2] for item in pos.items()])
        z_min = np.min([item[1][2] for item in pos.items()])
            
        ax.set_axis_off()
        ax.set_box_aspect([x_max - x_min, y_max - y_min, z_max - z_min])
        ax.set_xlim(x_min, x_max)
        ax.set_ylim(y_min, y_max)
        ax.set_zlim(z_min, z_max)
        plt.tight_layout()
        
        plt.show()



from setuptools import setup, find_packages
import re

# version
def get_version():
    with open("smiles2mol/__init__.py") as f:
        match = re.search(r'__version__ = "([^"]+)"', f.read())
        if match:
            return match.group(1)
        raise RuntimeError("Version not found in smiles2mol/__init__.py")

# long description
with open("README.md", "r") as f:
    long_description = f.read()
    
# install requires
with open("requirements.txt", "r") as f:
    install_requires = f.readlines()

setup(
    name="smiles2mol",
    version=get_version(),
    description="Chemical structure generator from SMILES strings.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Shota Inoue",
    entry_points={
        "console_scripts": [
            "smiles2mol = smiles2mol.main:main",
        ],
    },
    url="https://github.com/s-inoue0108/smiles2mol",
    license="MIT",
    include_package_data=True,
    packages=find_packages(),
    install_requires=install_requires,
    python_requires=">= 3.10, < 3.12",
)




import os
import numpy as np
from rdkit import Chem
from rdkit.Chem import AllChem
import networkx as nx

class Structure:
    def __init__(self, rdmol):
        self.rdmol = rdmol
        
    def generate3Drdmol(self, nconfs=10, seed=1):
        params = AllChem.ETKDGv3()
        params.randomSeed = seed
        rdmol = self.rdmol

        confs = AllChem.EmbedMultipleConfs(rdmol, numConfs=nconfs, params=params)

        # Optimization with MMFF
        for cid in confs:
            AllChem.MMFFOptimizeMolecule(rdmol, confId=cid)

        # sort with energy
        props = []
        for cid in confs:
            mmff = AllChem.MMFFGetMoleculeForceField(rdmol, AllChem.MMFFGetMoleculeProperties(rdmol), confId=cid)
            props.append((cid, mmff.CalcEnergy()))
        props.sort(key=lambda x: x[1])
        best_cid = props[0][0]
        
        # generate rdmol object
        mol_cid = Chem.Mol(rdmol)
        best_conf = rdmol.GetConformer(best_cid)
        mol_cid.RemoveAllConformers()
        mol_cid.AddConformer(best_conf, assignId=True)
        return mol_cid

    def get_coords(self, rdmol):
        coords = []
        conf = rdmol.GetConformer()
        for atom in rdmol.GetAtoms():
            idx = atom.GetIdx()
            symbol = atom.GetSymbol()
            pos = conf.GetAtomPosition(idx)
            coords.append([symbol, pos.x, pos.y, pos.z])
        return coords
        
    def get_bonds(self, rdmol):
        bonds = []
        for bond in rdmol.GetBonds():
            a1 = bond.GetBeginAtomIdx()
            a2 = bond.GetEndAtomIdx()
            order = bond.GetBondTypeAsDouble()
            bonds.append([order, (a1, a2)])
        return bonds
    
    def get_xyzstr(self, rdmol):
        coords = self.get_coords(rdmol)
        strs = ["\t".join([c[0], f"{c[1]:.6f}", f"{c[2]:.6f}", f"{c[3]:.6f}"]) for c in coords]
        return "\n".join([f"{len(coords)}", ""] + strs)
    
    def get_graph(self, rdmol):
        graph = nx.Graph()
        coords = self.get_coords(rdmol)
        bonds = self.get_bonds(rdmol)
        
        pos_list = np.array([[c[1], c[2], c[3]] for c in coords])

        # add node
        for i, (s, x, y, z) in enumerate(coords):
            symbol = rdmol.GetAtomWithIdx(i).GetSymbol()
            graph.add_node(i, symbol=s, pos=np.array([x, y, z]))

        # add edge
        nodes = graph.nodes(data=True)
        for b in bonds:
            i = b[1][0]
            j = b[1][1]
            dist = np.linalg.norm(graph.nodes[i]["pos"] - graph.nodes[j]["pos"])
            graph.add_edge(i, j, distance=dist, order=b[0])
        return graph
        
